{% extends "admin/app_index.html" %}

{% block content %}
{{ block.super }}
{% url 'defender_blocks_view' as blocks_url %}
{% if blocks_url %}
<div class="app-defender module">
    <table><tr scope='row'><td colspan='3'>
    <h4><a href='{{ blocks_url }}'>Blocked Users</a></h4>
    </td></tr></table>
    </div>
{% endif %}
{% endblock content%}
